/*
** Copyright (c) 2002, 2003
**      Jeff Forys (jeff@forys.cranbury.nj.us).  All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that: (1) Redistributions of
** source code must retain the above copyright notice, this list of
** conditions and the following disclaimer, (2) Redistributions in
** binary form must reproduce the above copyright notice, this list
** of conditions and the following disclaimer in the documentation
** and/or other materials provided with the distribution, (3) All
** advertising materials mentioning features or use of this software
** must display the following acknowledgment: ``This product includes
** software developed by Jeff Forys (jeff@forys.cranbury.nj.us).'', (4)
** The name of the author may not be used to endorse or promote products
** derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
** WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*/

const char *Version = "1.3 20030709";
const char *CopyrightVersion = "cammnt %s%s\n\nCopyright (c) 2002, 2003 Jeff Forys.  All rights reserved.\n\nRedistribution and use in source and binary forms, with or without\nmodification, is permitted under the terms specified in the source\ncode (and manual pages).\n";

#ifndef lint
static char rcsid[] = "$Id: cammnt.c,v 1.10 2003/07/10 00:52:21 forys Exp $";
#endif

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <syslog.h>
#include <pwd.h>

#include "config.h"
#include "compat.h"

int	version(void);

#ifdef	MNTDIR

/*
 * === CONFIGURE: MountArgs
 *
 * MountArgs[] is a command to mount a file system (e.g., MFS).
 * You will almost certainly have to tailor it to your machine.
 *
 * In this example, the file system is 384 sectors in size, mounted
 * at $MNTDIR, and backed by the swap partition "/dev/ad0s2b":
 *
 *	"/sbin/mount_mfs", "-s", "384", "/dev/ad0s2b", MNTDIR, NULL
 *
 * UmountArgs[] is a command to forcibly unmount said file system.
 * It may not need to be changed.
 */
char *const MountArgs[] = {
	"/sbin/mount_mfs", "-s", "384", "/dev/ad0s2b", MNTDIR, NULL
};

char *const UmountArgs[] = {
	"/sbin/umount", "-f", MNTDIR, NULL
};

int	usage(const char *, const char *);

/*
 * cammnt - verify user and execute mount/unmount command.
 *
 * cammnt [-u]
 */
int
main(int argc, char *argv[])
{
	char *const *av = MountArgs;
	const char *imgdir = MNTDIR;
	const char *mntpnt;
	struct passwd *pw;
	uid_t myuid;
	char *progname;
	int ch;

	SETPROGNAME(progname, argv);

	/* Parse flags, return usage() error if not defined */
	while ((ch = getopt(argc, argv, "uv")) != -1)
		switch (ch) {
		    case 'u':
			av = UmountArgs;
			break;
		    case 'v':
			return version();
		    case '?':
		    default:
			return usage(progname, "unknown flag");
                }
        argc -= optind;
        argv += optind;

	if (argc != 0)
		return usage(progname, "unexpected argument");

	openlog(progname, LOG_NDELAY | LOG_PID | LOG_PERROR, SYSLOGFAC);

	/* Ensure user has permission to do mount/unmount */
	if ((myuid = getuid()) != 0) {
		if ((pw = getpwuid(myuid)) == NULL) {
			syslog(LOG_ERR, "getpwuid: no such user: %d", myuid);
			return 2;
		}
		if (strcmp(pw->pw_name, CAMOWN) != 0) {
			errno = EPERM;
			syslog(LOG_ERR, "user %s: %s: %m", pw->pw_name, av[0]);
			return 1;
		}
		if (geteuid() != 0) {
			syslog(LOG_ERR, "installation error: not setuid root");
			return 2;
		}
	}

#ifdef	USE_SETREUID
	setreuid(geteuid(), -1);
	setregid(getegid(), -1);
#else
	setuid(geteuid());
	setgid(getegid());
#endif

	if ((mntpnt = getmntpnt(imgdir)) == NULL)
		return 2;

	/* If already mounted or unmounted, return successful */
	ch = strcmp(imgdir, mntpnt);
	if ((av == MountArgs && ch == 0) ||
			(av == UmountArgs && ch != 0))   
		return 0;

	/* Exec the command (no environment vars are passed) */
	execve(av[0], av, NULL);
	syslog(LOG_ERR, "execve(%s): %m", av[0]);
	return 2;
}

/*
 * Print usage info to STDERR and return '1'.
 */
int
usage(const char *progname, const char *errmsg)
{
	fprintf(stderr, "%s: parse error: %s\n", progname, errmsg);
	fprintf(stderr, "%s [-uv]\n", progname);
	return 1;
}

#else	/* !MNTDIR */

/*
 * cammnt - squawk and exit (not compiled with MNTDIR defined)
 *
 * cammnt [-u]
 */
int
main(int argc, char *argv[])
{
	char *progname;

	SETPROGNAME(progname, argv);

	if (argc == 2 && strcmp(argv[1], "-v") == 0)
		return version();

	openlog(progname, LOG_NDELAY | LOG_PID | LOG_PERROR, SYSLOGFAC);

	syslog(LOG_ERR, "administratively disabled: MNTDIR not defined");
	return 2;
}

#endif

int
version(void)
{
	printf(CopyrightVersion, Version,
#ifdef MNTDIR
					" +MNTDIR"
#else
					""
#endif
					);
	return 0;
}
